package ssw.annotation.AnnotationFileHandler;

/*
 * Created on 05.03.2005
 */
/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 * 
 * This class represents annotation items. It implements the interface AnnotationItem 
 * and contains the data of an annotation in the concrete use case.
 *  
 * 		checked: determines if the annotation was marked as reviewed.
 * 		description: the text of the annotation
 * 		file: file the annotation corresponds to
 * 		line: the line the annotated area begins in file
 * 		offset: the offset the annotated area begins at (whole file offset)
 * 		length: the length of the annotated area
 * 		points: the point modification value
 * 
 * For each of these attributes getter and setter methods are implemented. Additionally 
 * the data can be converted into a CSV string. When creating this string the description 
 * is modified with escape sequences. Line- and CSV separators are escaped. Moreover the 
 * escape character is escaped also. 
 * The attributes can be initialized using a CSV line, which either is passed to the 
 * constructor or to an instance method.
 * 
 * @version 1.0.0
 * @author Loidl Stefan, 0255886
 */

public class AnnoObject implements AnnotationItem{
	//Data fields
	private boolean checked=false;
	private String description;
	private int points;

	//Position fields
	private String file;
	private int line;
	private int offset;
	private int length;
	
	/**
	 * Constructor
	 * 
	 * @param CSVLine
	 */
	public AnnoObject(String CSVLine){
		setFromCSV(CSVLine);
	};
	
	/**
	 * Constructor
	 * @param checked
	 * @param desc
	 * @param points
	 * @param file
	 * @param line
	 * @param offset
	 * @param length
	 */
	public AnnoObject(boolean checked,String desc,int points,String file,int line,int offset, int length){
		this.description=desc;
		this.file=file;
		this.line=line;
		this.offset=offset;
		this.points=points;
		this.checked=checked;
		this.length=length;
	}

	/**
	 * 
	 * @uml.property name="description"
	 */
	//************Getter Methods************
	public String getDescription() {
		return description;
	}

	/**
	 * 
	 * @uml.property name="file"
	 */
	public String getFile() {
		return file;
	}

	/**
	 * 
	 * @uml.property name="points"
	 */
	public int getPoints() {
		return points;
	}

	/**
	 * 
	 * @uml.property name="checked"
	 */
	public boolean isChecked() {
		return checked;
	}

	/**
	 * 
	 * @uml.property name="offset"
	 */
	public int getOffset() {
		return offset;
	}

	/**
	 * 
	 * @uml.property name="length"
	 */
	public int getLength() {
		return length;
	}

	/**
	 * 
	 * @uml.property name="line"
	 */
	public int getLine() {
		return line;
	}

	/**
	 * 
	 * @uml.property name="description"
	 */
	//*************Setter Methoden***********
	public void setDescription(String desc) {
		description = desc;
	}

	/**
	 * 
	 * @uml.property name="file"
	 */
	public void setFile(String file) {
		this.file = file;
	}

	/**
	 * 
	 * @uml.property name="points"
	 */
	public void setPoints(int p) {
		this.points = p;
	}

	/**
	 * 
	 * @uml.property name="checked"
	 */
	public void setChecked(boolean ch) {
		this.checked = ch;
	}

	/**
	 * 
	 * @uml.property name="offset"
	 */
	public void setOffset(int o) {
		offset = o;
	}

	/**
	 * 
	 * @uml.property name="length"
	 */
	public void setLength(int l) {
		length = l;
	}

	/**
	 * 
	 * @uml.property name="line"
	 */
	public void setLine(int l) {
		line = l;
	}

	
	/**
	 * Generates a CSV line from the internal data.
	 * @return CSV line
	 */
	public String getCSVLine() {
		char seperator=AnnotationFile.CSVSeperator;
		char escChar=AnnotationFile.ESCAPEChar;
		char lineSep=AnnotationFile.CSVlineSep;
		String rlineSep= System.getProperty("line.separator");
		
		StringBuffer ESCDesc=new StringBuffer(description);
		
		//Escape the escape character, line- and csv seperator
		for(int i=0; i<ESCDesc.length();i++){
			if(ESCDesc.charAt(i)==escChar) {
				ESCDesc.insert(i,escChar);
				i++;
			}
			if(ESCDesc.charAt(i)==seperator) {
				ESCDesc.insert(i,escChar);
				i++;
			}
			if(ESCDesc.charAt(i)==lineSep) {
				ESCDesc.insert(i,escChar);
				i++;
			}
		}
		
		String desc=ESCDesc.toString();
		desc=desc.replaceAll(rlineSep,""+lineSep);
		
		return ""+checked+seperator+desc+seperator+points+
			seperator+file+seperator+line+seperator+offset+seperator+length;
	}

	/**
	 * Parses the values from a CSV line
	 * @param csv
	 * @return Success?
	 */
	public boolean setFromCSV(String csv) {
		char seperator=AnnotationFile.CSVSeperator;
		char escChar=AnnotationFile.ESCAPEChar;
		char lineSep=AnnotationFile.CSVlineSep;
		String rlineSep= System.getProperty("line.separator");
		
		//checked value
		int i=csv.indexOf(seperator);
		if(i==-1) return false;
		checked=Boolean.valueOf(csv.substring(0,i)).booleanValue();
		
		//description
		int j=i+1;
		
		//parse descripton excluding escaped seperators
		while(j< csv.length()){
			if(csv.charAt(j)==escChar) j+=2;
			else{
				if(csv.charAt(j)==seperator) break;
				j++;	
			}
		}
		
		if(j> csv.length()) return false;
		StringBuffer desc=new StringBuffer(csv.substring(i+1,j));

		for(int actual=0;actual<desc.length();actual++){
			if(desc.charAt(actual)==escChar){
				if(desc.length()<actual+1) return false; //Error- Esc Char at end of String
				char temp=desc.charAt(actual+1);
				desc.deleteCharAt(actual+1);
				if(temp==escChar||temp==seperator||temp==lineSep)
					desc.setCharAt(actual,temp);
				else return false; 						//Error- Invalid Escape sequence
			}else if(desc.charAt(actual)==lineSep){
				desc=desc.replace(actual,actual+1,rlineSep);
			}
		}
		
		description=desc.toString();
			
		//points
		i=j;
		j=csv.indexOf(seperator,i+1);
		if(j==-1) return false;
		points=Integer.valueOf(csv.substring(i+1,j)).intValue();

		//file
		i=j;
		j=csv.indexOf(seperator,i+1);
		if(j==-1) return false;
		file=csv.substring(i+1,j);

		//line
		i=j;
		j=csv.indexOf(seperator,i+1);
		if(j==-1) return false;
		line=Integer.valueOf(csv.substring(i+1,j)).intValue();
	
		//column
		i=j;
		j=csv.indexOf(seperator,i+1);
		if(j==-1) return false;
		offset=Integer.valueOf(csv.substring(i+1,j)).intValue();

		//length
		length=Integer.valueOf(csv.substring(j+1)).intValue();

		return true;
	}
}
