package ssw.annotation.annoedit;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import ssw.annotation.AnnotationPlugin;
import ssw.annotation.AnnotationFileHandler.AnnoObject;

/*
 * Created on 14.02.2005
 */
/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 *   
 *   
 * This class is the label provider for the annotation list of the annotation editor. 
 * The contentProvider provides AnnoObject objects which capsulate information about 
 * the description, file, if the annotation was marked as reviewed, the point 
 * modification, the highlighted line, offset and length of an annotation. The editor 
 * uses the information about description, point modification and file.
 * The description is a string and thus could contain line separators, which would 
 * not be displayed by the table in a correct way. These line separators are replaced 
 * with blanks.
 * If the provider is queried for an invalid column, a default string is returned.
 * 
 * @author Loidl Stefan, 0255886 
 */
public class AnnoEditLabelProvider extends LabelProvider implements ITableLabelProvider {
	
	//Used to replace lineseperators in the label
	private static final String BLANK=" ";
	private String DEFAULT;
	private String LINESEP=System.getProperty("line.separator");
	
	/**
	 * Constructor
	 */
	public AnnoEditLabelProvider(){
		super();
		DEFAULT=AnnotationPlugin.getResourceString("AE_LP_DEFAULT");	
	}
	
	/**
	 * Returns the label shown in the tabel.
	 */
	public String getColumnText(Object obj, int index) {
		switch(index){
			case 0: 
				//clone description string
				String desc=((AnnoObject)obj).getDescription().substring(0);
				desc=desc.replaceAll(LINESEP,BLANK);
				return desc;
			case 1: return String.valueOf(((AnnoObject)obj).getPoints());
			case 2: return ((AnnoObject)obj).getFile();
			default: return DEFAULT;
		}
	}
	
	/**
	 * Returns the image shown at the beginning of each field.
	 * No image is provided here!
	 */
	public Image getColumnImage(Object obj, int index) {
		return null;
	}	
}
