package ssw.annotation.annoview.popup.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import ssw.annotation.annoview.dialogs.AnnoInputDialog;

/*
 * Created on 08.03.2005
 */
/**
 *   Copyright (C) 2005  Stefan Loidl
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *   
 *   eMail: stefanloidl@gmx.net
 *   
 * This class implements the action, performed if the button from the context menu of 
 * the compilation unit editor, to create an annotation, is pressed. The menu entry 
 * is added to the context menu using the plugin.xml file, which is done using the 
 * popupMenus extension point. 
 * The reference to this class is defined during the initialization of this extension point.
 * The run method of the action class is overwritten to retrieve the information of the 
 * file to be annotated. This information is passed to a new AnnoInputDialog which 
 * is opened to collect further annotation information.                   
 * 
 * @version 1.0.0
 * @author Loidl Stefan, 0255886
 */
public class ActionAnnotation implements IEditorActionDelegate {

	private char ECL_FILESEP='/';
	
	/**
	 * Constructor
	 */
	public ActionAnnotation() {
		super();
	}

	/**
	 * Action run method
	 */
	public void run(IAction action) {
		//shell for AnnoInputDialog
		Shell shell = new Shell();
		//Extract die file/project
		IWorkbenchPage wbPage = PlatformUI.getWorkbench().
								getActiveWorkbenchWindow().getActivePage();
		IEditorInput input=wbPage.getActiveEditor().getEditorInput();
		IFile file= (IFile)input.getAdapter(IFile.class);
		String filepath=file.getFullPath().toString();
		int i=filepath.indexOf(ECL_FILESEP,1);
		
		if(i!=-1){
			String project=filepath.substring(1,i);
			filepath=filepath.substring(i);
			//create dialog
			AnnoInputDialog d=new AnnoInputDialog(shell,filepath,project);
			d.open();
		}
	}
	
	/**
	 * Not used.
	 */
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {}

	/**
	 * Not used.
	 */
	public void selectionChanged(IAction action, ISelection selection) {}

}
